using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Events;

namespace Damage
{
    public class HumanDamageBehaviour : MonoBehaviour
    {
        private LimbBehaviour limb;
        private PersonBehaviour person;

        //[SerializeField]
        public Organ[] organs;

        //[SerializeField]
        //public bool spawned = false;

        //[SerializeField]
        public bool hasOrgan;
        //[SerializeField]
        public bool fracturedBone;
        //[SerializeField]
        public float boneHealth = 50f;
        //[SerializeField]
        public float initialBoneHealth;
        //[SerializeField]
        public float fractureChance = 0.75f;

        //[SerializeField]
        public float tissueHealth;
        //[SerializeField]
        public float tissueInitialHealth;
        public float tissueDamageMultiplier;
        float shotHeat;
        public bool vitalOrganDestroyed;
        public float strength;

        private ContactPoint2D[] contactBuffer = new ContactPoint2D[8];

        bool CanRevive()
        {
            int deadNum = 0;
            foreach (LimbBehaviour limb in person.Limbs)
            {
                HumanDamageBehaviour damage = limb.GetComponent<HumanDamageBehaviour>();
                if (damage.hasOrgan)
                {
                    foreach (Organ organ in damage.organs)
                    {
                        if (organ.vital && organ.destroyed)
                        {
                            deadNum++;
                        }
                    }
                }
            }
            if (deadNum == 0)
                return true;
            else
                return false;
        }

        
        void Start()
        {
            limb = gameObject.GetComponent<LimbBehaviour>();
            person = limb.Person;
            strength = limb.BaseStrength;

            Organ brain = ScriptableObject.CreateInstance<Organ>();
            brain.name = "Brain";
            brain.health = 20f;
            brain.initialHealth = 20f;
            brain.vital = true;
            brain.chance = .99f;
            brain.damageMultiplier = 3f;

            Organ heart = ScriptableObject.CreateInstance<Organ>();
            heart.name = "Heart";
            heart.health = 80f;
            heart.initialHealth = 80f;
            heart.vital = true;
            heart.chance = .5f;

            Organ lungs = ScriptableObject.CreateInstance<Organ>();
            lungs.name = "Lungs";
            lungs.health = 50f;
            lungs.initialHealth = 50f;
            lungs.chance = .75f;

            switch (gameObject.name)
            {
                case "Head":
                organs = new Organ[]{
                    brain
                };
                hasOrgan = true;
                break;
                case "UpperBody":
                organs = new Organ[]{
                    heart,
                    lungs
                };
                fractureChance = .5f;
                hasOrgan = true;
                break;
                case "MiddleBody":
                fractureChance = .25f;
                boneHealth = 30f;
                break;
                case "LowerBody":
                fractureChance = .15f;
                boneHealth = 25f;
                break;
                case "UpperLegFront":
                boneHealth = 200f;
                break;
                case "UpperLeg":
                boneHealth = 200f;
                break;
            }
            switch (limb.RoughClassification)
            {
                case LimbBehaviour.BodyPart.Arms:
                tissueHealth = 250f;
                tissueDamageMultiplier = 0.1f;
                break;
                case LimbBehaviour.BodyPart.Legs:
                tissueHealth = 250f;
                tissueDamageMultiplier = 0.1f;
                break;
                case LimbBehaviour.BodyPart.Head:
                tissueHealth = 500f;
                tissueDamageMultiplier = 0.1f;
                break;
                case LimbBehaviour.BodyPart.Torso:
                tissueHealth = 800f;
                tissueDamageMultiplier = 0.1f;
                break;
            }
            initialBoneHealth = boneHealth;
            tissueInitialHealth = tissueHealth;
        }
        void Update()
        {
            if ((double) limb.CirculationBehaviour.TotalLiquidAmount > 1.40129846432482E-45)
            {
                foreach (KeyValuePair<Liquid, BloodContainer.RefFloat> keyValuePair in limb.CirculationBehaviour.LiquidDistribution)
                {
                    Liquid liquid = keyValuePair.Key;
                    string liquidID = Liquid.GetIdentity(liquid);
                    switch (liquidID)
                    {
                        case "INSTANT DEATH POISON":
                            KillBody();
                            break;
                        case "LIFE SERUM":
                            HealBone();
                            if (hasOrgan)
                            {
                                foreach (Organ organ in organs)
                                {
                                    RepairOrgan(organ);
                                }
                            }
                            tissueHealth = tissueInitialHealth;
                            break;
                        case "MENDING SERUM":
                            HealBone();
                            if (hasOrgan)
                            {
                                foreach (Organ organ in organs)
                                {
                                    organ.health = organ.initialHealth;
                                }
                            }
                            tissueHealth = tissueInitialHealth;
                            break;        
                        default:
                            break;
                    }
                }
            }     
            if (hasOrgan)
            {
                foreach (Organ organ in organs)
                {
                    if (!organ.destroyed)
                    {
                        if (organ.regenerationSpeed > 0 && organ.health < organ.initialHealth)
                        {
                            organ.health += (organ.regenerationSpeed * limb.CirculationBehaviour.BloodFlow) * Time.deltaTime;
                        }
                        if (limb.CirculationBehaviour.BloodFlow < 0.2f)
                        {
                            organ.Damage(Time.deltaTime);
                            //if (organ.destroyed)
                                //Debug.Log("Bleedout");
                        }
                        if (person.OxygenLevel <= 0)
                        {
                            organ.Damage(Time.deltaTime * 2);
                            //if (organ.destroyed)
                                //Debug.Log("Suffocation");
                        }
                        if (limb.PhysicalBehaviour.Temperature >= limb.DiscomfortingHeatTemperature || limb.PhysicalBehaviour.Temperature <= limb.BodyTemperature - 30)
                            organ.Damage(3f/1000f);
                        if (limb.PhysicalBehaviour.Temperature > limb.PhysicalBehaviour.Properties.BurningTemperatureThreshold || limb.PhysicalBehaviour.Temperature < limb.FreezingTemperature)
                            organ.Damage(0.5f);
                    }
                    if (organ.vital)
                    {
                        limb.Health = organ.health;
                        if (!organ.destroyed)
                        {
                            if (limb.IsDismembered)
                                KillBody(); 
                            if (limb.IsLethalToBreak && limb.Broken)
                                KillBody();
                        }
                        if (organ.destroyed || vitalOrganDestroyed)
                        {
                            limb.CirculationBehaviour.IsPump = false;
                            if (!vitalOrganDestroyed)
                                KillBody();
                            vitalOrganDestroyed = true;
                        }
                        else
                        {
                            limb.CirculationBehaviour.IsPump = true;
                            vitalOrganDestroyed = false;
                        }
                    }
                    if (organ.health < organ.initialHealth)
                    {
                        if (organ.name == "Lungs")
                        {
                            // cancel out the regeneration of oxygen
                            // just copied straight from dotpeek lol
                            person.OxygenLevel -= (float) ((double) Mathf.Pow(Mathf.Sin(Time.time * 3f), 16f) * 0.5 / 90.0 * ((double) Time.deltaTime * 60.0));
                            // remove oxygen
                            person.OxygenLevel -= Time.deltaTime / Mathf.Clamp(360 * (organ.health / organ.initialHealth), 60, 360);
                        }
                    }
                    organ.health = Mathf.Clamp(organ.health, 0, organ.initialHealth);  
                }
            }
        }
        void FixedUpdate()
        {
            shotHeat = 1f;
        }
        public void KillBody()
        {
            //Debug.Log("Death");
            foreach (LimbBehaviour limb in limb.Person.Limbs)
            {
                HumanDamageBehaviour damageBehaviour = limb.GetComponent<HumanDamageBehaviour>();
                damageBehaviour.vitalOrganDestroyed = true;
                if (damageBehaviour.hasOrgan)
                {
                    foreach (Organ organ in damageBehaviour.organs)
                    {
                        organ.health = 0;
                        organ.destroyed = true;
                    }
                }
            }
        }
        private void OnFragmentHit(float force)
        {
            float fracturerng = UnityEngine.Random.value;
            if (fracturerng < fractureChance)
                FractureBone(force * 2f);
            if (hasOrgan)
            {
                foreach (Organ organ in organs)
                {
                    float rng = UnityEngine.Random.value;
                    DamageOrgan(force * 5f, organ, false, rng);
                }
            }
            //Debug.Log("Fragment hit with force of " + force);
        }
        private void Stabbed(Stabbing stab)
        {
            if (!stab.stabber.StabCausesWound)
                return;
            float artery = UnityEngine.Random.value;
            if (hasOrgan)
            {
                foreach (Organ organ in organs)
                {
                    float damage = UnityEngine.Random.Range(0, organ.initialHealth * 0.5f);
                    float rng = UnityEngine.Random.value * 0.85f;
                    if (organ.name == "Brain")
                        damage = 50f;
                    DamageOrgan(damage, organ, false, rng);
                }
            }
            if (limb.RoughClassification == LimbBehaviour.BodyPart.Arms && artery < 0.3f)
                HitArtery(0.5f);
            if (limb.RoughClassification == LimbBehaviour.BodyPart.Legs && artery < 0.2f)
                HitArtery(1f);
        }
        private void Shot(global::Shot shot)
        {
            shotHeat += 2;
            float damage = shot.damage;
            float fracturerng = UnityEngine.Random.value;
            float fragmentrng = UnityEngine.Random.value;
            if (!fracturedBone && fracturerng < fractureChance)
                FractureBone(damage);
            if (damage > 10 && UnityEngine.Random.value < 0.75f)
            {
                damage *= UnityEngine.Random.Range(1f, 1.5f);
                //Debug.Log("Yawing");
            }
            if (hasOrgan)
            {
                foreach (Organ organ in organs)
                {
                    if (organ.name == "Brain")
                        person.Consciousness = 0f;
                    float organrng = UnityEngine.Random.value;
                    DamageOrgan(damage, organ, false, organrng);
                }
            }
            foreach (LimbBehaviour otherLimb in limb.ConnectedLimbs)
            {
                float rng = UnityEngine.Random.value;
                if (limb.Health <= 0)
                    otherLimb.Damage(shot.damage / 5);
                if (rng < 0.2f)
                {   
                    if (limb.Health > 0)
                        otherLimb.Damage(shot.damage / 20);
                    if (otherLimb.RoughClassification == limb.RoughClassification && otherLimb.gameObject.GetComponent<HumanDamageBehaviour>().hasOrgan)
                    {
                        foreach (Organ organ in otherLimb.gameObject.GetComponent<HumanDamageBehaviour>().organs)
                        {
                            float organrng = UnityEngine.Random.value;
                            DamageOrgan(damage / 5, organ, false, organrng);
                        }
                    }
                }
            }
            if (fragmentrng < 0.05f)
            {
                FragmentDamage(UnityEngine.Random.Range(2, 3), Mathf.Clamp(damage / 5, 0, 25));
                //Debug.Log("Bullet fragmentation");
            }
            if (shot.damage >= (tissueHealth / shotHeat))
                DamageTissue(shot.damage, 1);
            float artery = UnityEngine.Random.value;
            if (limb.RoughClassification == LimbBehaviour.BodyPart.Arms && artery < 0.3f)
                HitArtery(0.5f);
            if (limb.RoughClassification == LimbBehaviour.BodyPart.Legs && artery < 0.2f)
                HitArtery(1f);
            StartCoroutine(EnterShock(20f));
        }
        public IEnumerator EnterShock(float time)
        {
            while(time > 0)
            {
                limb.Person.AdrenalineLevel = 20f;
                time -= Time.deltaTime;
                yield return null;
            }
        }
        private void ExitShot(global::Shot shot)
        {
            if (shot.damage >= (tissueHealth / shotHeat))
                DamageTissue(shot.damage, 2);
        }
        public void DamageTissue(float damage, float multiplier)
        {
            damage *= multiplier * shotHeat;
            tissueHealth -= damage * tissueDamageMultiplier;
            //Debug.Log("Tissue damaged by " + damage * tissueDamageMultiplier + " damage. Tissue health now at " + tissueHealth);
            if (tissueHealth <= 0)
            {
                limb.BreakingThreshold = 0.0f;
                //Debug.Log("Dismemberment");
            }
        }
        public void HitArtery(float severity)
        {
            limb.CirculationBehaviour.BloodLossRateMultiplier += severity;
            //Debug.Log("Artery broken");
        }
        public void FractureBone(float damage)
        {
            if (boneHealth <= 0)
                return;
            if (damage < 6)
            {
                limb.Damage(damage);
                return;
            }
            float random = UnityEngine.Random.value;
            // just guessed these numbers
            int fragmentCount = UnityEngine.Random.Range(5, 18);

            if (random < fractureChance || damage < 10)
            {
                float fragmentDamage = Mathf.Clamp(damage, 0, initialBoneHealth / 5);
                limb.BaseStrength /= 2;
                FragmentDamage(fragmentCount, fragmentDamage / fragmentCount);
                boneHealth -= fragmentDamage;
                //Debug.Log("Fractured bone. " + "Health at " + boneHealth);
            }
            else
            {
                limb.BaseStrength = 0;
                float previousBoneHealth = boneHealth;
                boneHealth -= 100;
                boneHealth = Mathf.Clamp(boneHealth, 0, Mathf.Infinity);
                float fragmentDamage = previousBoneHealth - boneHealth;
                float count = fragmentCount * UnityEngine.Random.Range(1, 2);
                FragmentDamage(count, fragmentDamage / count);
                //Debug.Log("Shattered bone. " + "Health at " + boneHealth);
            }
            boneHealth = Mathf.Clamp(boneHealth, 0, Mathf.Infinity);
        }
        public void FragmentDamage(float amount, float damage)
        {
            int neighboringLimbs = limb.ConnectedLimbs.Count;
            amount /= neighboringLimbs;
            foreach (LimbBehaviour otherLimb in limb.ConnectedLimbs)
            {
                otherLimb.Damage(damage * amount);

                if (otherLimb.TryGetComponent<HumanDamageBehaviour>(out HumanDamageBehaviour damageBehaviour))
                {
                    if (damageBehaviour.hasOrgan)
                    {
                        foreach (Organ organ in damageBehaviour.organs)
                        {
                            float rng = UnityEngine.Random.value;
                            damageBehaviour.DamageOrgan((damage * amount) * rng, organ, true, 0);
                        }
                    }
                }
            }
        }
        public void DamageOrgan(float damage, Organ organ, bool ignoreMultiplier, float rng)
        { 
            if (!ignoreMultiplier)
                damage *= organ.damageMultiplier;
            if (!organ.destroyed && rng < organ.chance)
            {
                organ.Damage(damage);
                //Debug.Log(organ.name + " damaged by " + damage + ". Health is now " + organ.health);
                //if (organ.destroyed)
                    //Debug.Log(organ.name + " destroyed.");
            }
        }
        public void HealBone()
        {
            //Debug.Log("Bone healed.");
            boneHealth = initialBoneHealth;
            limb.BaseStrength = strength;
        }
        public void RepairOrgan(Organ organ)
        {
            //Debug.Log(organ.name + " repaired.");
            organ.health = organ.initialHealth;
            if (organ.vital)
            {
                if (vitalOrganDestroyed && CanRevive())
                    Revive();
                vitalOrganDestroyed = false;
            }
            organ.destroyed = false;
        }
        public void Revive()
        {
            foreach (LimbBehaviour limb in person.Limbs)
            {
                HumanDamageBehaviour damage = limb.GetComponent<HumanDamageBehaviour>();
                if (damage)
                {
                    damage.vitalOrganDestroyed = false;
                }
            }
        }
        void OnCollisionEnter2D(Collision2D collision)
        {
            // ctrl v
            float impulse = ((IEnumerable<ContactPoint2D>)this.contactBuffer).Take<ContactPoint2D>(collision.GetContacts(this.contactBuffer)).Max<ContactPoint2D>((Func<ContactPoint2D, float>)(c => c.normalImpulse));
            if (impulse > 1f)
            {
                float rng = UnityEngine.Random.value;
                float fracturerng = UnityEngine.Random.value;
                if (hasOrgan)
                {
                    foreach (Organ organ in organs)
                    {
                        DamageOrgan(impulse * 5, organ, true, rng * 0.8f);
                    }
                }
                if (fracturerng < .5f && impulse > 2f)
                {
                    FractureBone(impulse);
                }
            }
        }
    }

    [System.Serializable]
    public class Organ : ScriptableObject
    {
        public string name;
        public float health;
        public float initialHealth;
        public float chance;
        public float damageMultiplier = 1f;
        public bool destroyed;
        public bool vital;
        public float regenerationSpeed = 0f;

        public void Damage(float damage)
        {
            health -= damage;
            if (health <= 0)
                destroyed = true;
        }
    }
}