using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Events;

namespace Damage
{
    public class GorseDamageBehaviour : MonoBehaviour
    {
        // Organs are Acid Gland, Brain, and Lung
        public float acidHealth = 50;
        bool glandActive = true;
        public float brainHealth = 200;
        bool brainActive = true;

        // Breathing stuff
        public float lungHealth = 75;
        public float lungEfficiency = 1;
        // Variables
        public LimbBehaviour limb;
        public PersonBehaviour person;
        void Start()
        {
            StartCoroutine(SlowUpdate());
            limb = GetComponent<LimbBehaviour>();
            person = limb.Person;
        }
        void Update()
        {
            lungEfficiency = lungHealth / 75;

            if (GetComponent<LifePoison>())
            {
                Revive("Brain");
                Revive("Gland");
                Revive("Lung");
            }
            limb.Health = brainHealth;
            if (brainHealth > 0 && limb.CirculationBehaviour.IsPump == false)
                limb.CirculationBehaviour.IsPump = true;

            if (brainHealth <= 0 && brainActive)
                Shutdown("Brain");

            if (lungEfficiency >= 1)
                return;
            float oxygenLossRate = 120 * lungEfficiency;
            oxygenLossRate = Mathf.Clamp(oxygenLossRate, 6f, 120f);
            person.OxygenLevel -= Time.deltaTime / oxygenLossRate;
        }
        IEnumerator SlowUpdate()
        {
            while (1 == 1)
            {
                
                
                yield return new WaitForSeconds(.5f);
            }
        }
        public void Shutdown(string organ)
        {
            ModAPI.Notify(organ + " destroyed");
            switch (organ)
            {
                case "Brain":
                    brainHealth = 0;
                    limb.CirculationBehaviour.IsPump = false;
                    brainActive = false;
                    break;
                case "Gland":
                    glandActive = false;
                    acidHealth = 0;
                    break;
                case "Lung":
                    lungHealth = 0;
                    lungEfficiency = 0;
                    break;
            }
        }
        public void Revive(string organ)
        {
            switch (organ)
            {
                case "Brain":
                    brainHealth = 100;
                    limb.CirculationBehaviour.IsPump = true;
                    brainActive = true;
                    break;
                case "Gland":
                    acidHealth = 50;
                    glandActive = true;
                    break;
                case "Lung":
                    lungHealth = 75;
                    lungEfficiency = 1;
                    break;
            }
        }
        public void Damage(float damage, string organ)
        {
            switch (organ)
            {
                case "Brain":
                    brainHealth -= damage;
                    break;
                case "Gland":
                    acidHealth -= damage;
                    break;
                case "Lung":
                    lungHealth -= damage;
                    break;
            }
        }
        private void Shot(global:: Shot shot)
        {
            // Organs are Acid Gland, Brain, and Lung
            // Chance of hitting each (respectively) is 0.5, 0.2, and 0.7
            float acidrng = UnityEngine.Random.value;
            float brainrng = UnityEngine.Random.value;
            float lungrng = UnityEngine.Random.value;
            if (acidrng < .5f && acidHealth > 0)
                Damage(shot.damage, "Gland");
            if (brainrng < .2f && brainHealth > 0)
                Damage(shot.damage * 2, "Brain");
            if (lungrng < .7f && lungHealth > 0)
                Damage(shot.damage, "Lung");
        }
    }
}