using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Events;

namespace Damage
{
    public class UltraStrengthOrganSyringe : SyringeBehaviour
    {
        public override string GetLiquidID() => UltraStrengthOrganSerumLiquid.ID;

        public class UltraStrengthOrganSerumLiquid : TemporaryBodyLiquid
        {
            public const string ID = "ULTRA STRENGTH SERUM [Advanced Damage]";

            public UltraStrengthOrganSerumLiquid()
            {
                this.Color = new Color(0.7604228f, 0.7137255f, 1f);
            }

            public override void OnEnterContainer(BloodContainer container)
            {
            }

            public override void OnEnterLimb(LimbBehaviour limb)
            {
                if (limb.SpeciesIdentity == "Android")
                    return;
                limb.Vitality *= 0.1f;
                limb.RegenerationSpeed += 5f;
                limb.ImpactPainMultiplier *= 0.1f;
                limb.InitialHealth += 150f;
                limb.Health = limb.InitialHealth;
                limb.BreakingThreshold += 10f;
                limb.BaseStrength = Mathf.Min(15f, limb.BaseStrength + 5f);
                limb.ShotDamageMultiplier *= 0.1f;
                limb.CirculationBehaviour.HealBleeding();
                if (limb.gameObject.TryGetComponent<HumanDamageBehaviour>(out HumanDamageBehaviour damage))
                {
                    damage.strength = limb.BaseStrength;
                    damage.tissueInitialHealth += 150f;
                    damage.tissueHealth = damage.tissueInitialHealth;
                    damage.initialBoneHealth += 150f;
                    damage.boneHealth = damage.initialBoneHealth;
                    damage.fractureChance *= 0.75f;
                    if (damage.hasOrgan)
                    {
                        foreach (Organ organ in damage.organs)
                        {
                            organ.initialHealth += 50f;
                            organ.health = organ.initialHealth;
                            organ.damageMultiplier *= 0.8f;
                            organ.regenerationSpeed += 1f;
                        }
                    }
                }
                limb.Wince(10f);
            }

            public override void OnExitContainer(BloodContainer container)
            {
            }
        }
    }
}